@@ -61,6 +61,9 @@ class dataset(data_object):
         h5_file = dio.h5io.get_h5_filename(self.root_dir)
         if h5_file is None:
             h5_file = os.path.join(self.root_dir, '%s.h5' % self.data_name)
+            print(f'No existing h5 file found. New h5 will be created at {h5_file}.')
+        else:
+            print(f'Existing h5 file found. Using {h5_file}.')
 
         self.h5_file = h5_file
 
@@ -140,19 +143,24 @@ class dataset(data_object):
         spike_array_params['sampling_rate'] = sampling_rate
         clustering_params['file_dir'] = file_dir
         clustering_params['sampling_rate'] = sampling_rate
+        self.spike_array_params = spike_array_params
 
         # Setup digital input mapping
         if rec_info.get('dig_in'):
             self._setup_digital_mapping('in', dig_in_names, shell)
             dim = self.dig_in_mapping.copy()
-            spike_array_params['laser_channels'] = dim.channel[dim['laser']].to_list()
-            spike_array_params['dig_ins_to_use'] = dim.channel[dim['spike_array']].to_list()
         else:
             self.dig_in_mapping = None
 
         if rec_info.get('dig_out'):
-            self._setup_digital_mapping('out', dig_out_names, shell)
-            dom = self.dig_out_mapping.copy()
+            q = userIO.ask_user('Your info.rhd suggests you have digital '
+                                'outputs. Is this True?', shell=shell)
+            if q == 1:
+                self._setup_digital_mapping('out', dig_out_names, shell)
+                dom = self.dig_out_mapping.copy()
+            else:
+                _ = rec_info.pop('dig_out')
+                self.dig_out_mapping = None
         else:
             self.dig_out_mapping = None
 
@@ -164,7 +172,6 @@ class dataset(data_object):
         self._set_CAR_groups(group_keyword=car_keyword, group_areas=car_group_areas, shell=shell)
 
         # Confirm parameters
-        self.spike_array_params = spike_array_params
         if not accept_params:
             conf = userIO.confirm_parameter_dict
             clustering_params = conf(clustering_params,
@@ -265,7 +272,7 @@ class dataset(data_object):
         if len(em['CAR_group'].unique()) != len(areas):
             raise ValueError('Number of items in areas must match number of CAR groups')
 
-        em['areas'] = em['CAR_group'].apply(lambda x: areas[int(x)])
+        em['area'] = em['CAR_group'].apply(lambda x: areas[int(x)])
         self.electrode_mapping = em.copy()
         dio.h5io.write_electrode_map_to_h5(self.h5_file, self.electrode_mapping)
         self.save()
@@ -313,7 +320,11 @@ class dataset(data_object):
             df2['palatability_rank'] = df2['palatability_rank'].fillna(-1).astype('int')
 
         if dig_type == 'in':
-            self.dig_in_mapping = df2.copy()
+            self.dig_in_mapping = dim = df2.copy()
+            self.spike_array_params['laser_channels'] = dim.channel[dim['laser']].to_list()
+            self.spike_array_params['dig_ins_to_use'] = dim.channel[dim['spike_array']].to_list()
+            wt.write_params_to_json('spike_array_params', self.root_dir,
+                                    self.spike_array_params)
         else:
             self.dig_out_mapping = df2.copy()
 
@@ -378,7 +389,7 @@ class dataset(data_object):
             dim.loc[[x in tmp['dig_ins_to_use'] for x in dim.channel],
                     'spike_array'] = True
 
-        dim['laser_channels'] = False
+        dim['laser'] = False
         if tmp['laser_channels'] != ['']:
             tmp['laser_channels'] = [int(x) for x in tmp['laser_channels']]
             dim.loc[[x in tmp['laser_channels'] for x in dim.channel],
@@ -387,6 +398,10 @@ class dataset(data_object):
         self.spike_array_params = tmp.copy()
         wt.write_params_to_json('spike_array_params',
                                 self.root_dir, tmp)
+        if os.path.isfile(self.h5_file):
+            dio.h5io.write_digital_map_to_h5(self.h5_file, self.dig_in_mapping, 'in')
+
+        self.save()
 
     def edit_clustering_params(self, shell=False):
         '''Allows user interface for editing clustering parameters
@@ -403,6 +418,8 @@ class dataset(data_object):
             self.clustering_params = tmp
             wt.write_params_to_json('clustering_params', self.root_dir, tmp)
 
+        self.save()
+
     def edit_psth_params(self, shell=False):
         '''Allows user interface for editing psth parameters
 
@@ -418,6 +435,8 @@ class dataset(data_object):
             self.psth_params = tmp
             wt.write_params_to_json('psth_params', self.root_dir, tmp)
 
+        self.save()
+
     def edit_pal_id_params(self, shell=False):
         '''Allows user interface for editing palatability/identity parameters
 
@@ -433,6 +452,8 @@ class dataset(data_object):
             self.pal_id_params = tmp
             wt.write_params_to_json('pal_id_params', self.root_dir, tmp)
 
+        self.save()
+
     def __str__(self):
         '''Put all information about dataset in string format
 
@@ -764,7 +785,7 @@ class dataset(data_object):
                 n_cores = cpu_count() - 1
 
             results = Parallel(n_jobs=n_cores, verbose=10,
-                               backend='multiprocessing')(delayed(detect_spikes)
+                               backend='multiprocessing')(delayed(run_joblib_process)
                                                           (sd) for sd in spike_detectors)
             # results = Parallel(n_jobs=n_cores)(delayed(run_joblib_process)(sd)
             #                                    for sd in spike_detectors)
@@ -1111,19 +1132,11 @@ class dataset(data_object):
         self.make_psth_arrays()
 
 
-def detect_spikes(sd):
-    res = sd.run()
-    return res
-
-# def detect_spikes(data_dir, electrode, params, overwrite=False):
-#     sd = clust.SpikeDetection(data_dir, electrode, params=params, overwrite=overwrite)
-#     res = sd.run()
-#     return res
-
 def run_joblib_process(process):
     res = process.run()
     return res
 
+
 def port_in_dataset(rec_dir=None, shell=False):
     '''Import an existing dataset into this framework
     '''
@@ -1174,17 +1187,19 @@ def port_in_dataset(rec_dir=None, shell=False):
         return dat
 
     # write eletrode map and digital input & output maps to hf5
-    dio.h5io.write_electrode_map_to_h5(dat.h5_file, dat.electrode_mapping)
-    if dat.rec_info.get('dig_in') is not None:
+    node_list = dio.h5io.get_node_list(dat.h5_file)
+
+    if 'electrode_map' not in node_list:
+        dio.h5io.write_electrode_map_to_h5(dat.h5_file, dat.electrode_mapping)
+
+    if dat.rec_info.get('dig_in') is not None and 'digital_input_map' not in node_list:
         dio.h5io.write_digital_map_to_h5(dat.h5_file, dat.dig_in_mapping, 'in')
 
-    if dat.rec_info.get('dig_out') is not None:
+    if dat.rec_info.get('dig_out') is not None and 'digital_output_map' not in node_list:
         dio.h5io.write_digital_map_to_h5(dat.h5_file, dat.dig_out_mapping, 'out')
 
 
-    node_list = dio.h5io.get_node_list(dat.h5_file)
-
-    if (status['create_trial_list'] == False) and ('digital_in' in node_list):
+    if ('trial_info' not in node_list) and ('digital_in' in node_list):
         dat.create_trial_list()
     else:
         status['create_trial_list'] == True
@@ -1197,8 +1212,9 @@ def port_in_dataset(rec_dir=None, shell=False):
         params = dat.spike_array_params
         array_time = np.arange(-params['pre_stimulus'], params['post_stimulus'], 1)
         for x in digs:
-            dio.h5io.write_array_to_hdf5(dat.h5_file, f'/spike_trains/{x}',
-                                         'array_time', array_time)
+            if f'spike_trains.{x}.array_time' not in node_list:
+                dio.h5io.write_array_to_hdf5(dat.h5_file, f'/spike_trains/{x}',
+                                             'array_time', array_time)
 
         for x in dat.processing_steps:
             status[x] = True
